from __future__ import absolute_import
from argparse import ArgumentParser
from sokery.utils.types import portlist
from sokery.loop import start_loop

parser = ArgumentParser()
parser.add_argument('action')
parser.add_argument('-p', dest='port', default=4991, type=int)
parser.add_argument('-l', dest='listen', type=portlist)
parser.add_argument('-a', dest='allowed_ports', type=portlist)
parser.add_argument('--proxy-http', action='store_const',
					const=True, default=False, dest='proxy_http')

args = parser.parse_args()

if args.action == 'run':
	try:
		print('listening on port {}'.format(args.port))
		start_loop(port=args.port,
				   listen_to=args.listen,
				   proxy_http=args.proxy_http,
				   allowed_ports=args.allowed_ports,
				   )
	except KeyboardInterrupt:
		print('end of loop')


import signal
import socket
import argparse
import time


class Global:
    run = True

parser = argparse.ArgumentParser()
parser.add_argument('host', nargs=1)
parser.add_argument('port', nargs=1, type=int)
parser.add_argument('-p', dest='payload', type=str, default='hello')
parser.add_argument('-t', dest='tick', type=float, default=2.0)
parser.add_argument('--br', dest='line_break', default=False,
                            action='store_const', const=True)
args = parser.parse_args()
address = (args.host[0], args.port[0], )

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.connect(address)

while Global.run:
    try:
        s.send(args.payload)
        if args.line_break:
            s.send('\n')
        print('sending: {}'.format(args.payload))
        time.sleep(args.tick)

    except KeyboardInterrupt:
        break

    except socket.error as ex:
        print(ex)
        break


import json
import tornado.gen
import tornado.tcpserver
from tornado.iostream import StreamClosedError
from binascii import hexlify
import os


class TCPServer(tornado.tcpserver.TCPServer):

    def __init__(self, *args, **kwargs):
        super(TCPServer, self).__init__(*args, **kwargs)
        self.report_clients = None
        self.uuid = hexlify(os.urandom(8)).decode('utf8')
        self.is_hex_digest = True

    def listen(self, port):
        self.port = port
        self.echo_all = False
        super(TCPServer, self).listen(port)

    @tornado.gen.coroutine
    def handle_stream(self, stream, address):
        print('connection from {}'.format(address))

        pkg = json.dumps({
            'op': 'newclient',
            'from': address + (self.port, ),
            })

        for client in self.report_clients:
            client.write_message(pkg)

        while True:
            try:
                data = yield stream.read_bytes(4096, partial=True)
                pkg = json.dumps({
                    'op': 'recvd',
                    'is_hex_digest': self.is_hex_digest,
                    'data': hexlify(data).decode('utf8') \
                            if self.is_hex_digest \
                            else data.decode('utf8'),
                    'from': address + (self.port, )
                })

                if self.echo_all:
                    yield stream.write(data)

                for client in self.report_clients:
                    client.write_message(pkg)

            except StreamClosedError:
                host, socket = address
                pkg = json.dumps({
                    'op': 'connection_closed',
                    'from': address + (self.port, )
                })
                for client in self.report_clients:
                    client.write_message(pkg)
                break



import sys

portrange = lambda up, low: (int(up), int(low))


def portlist(x):
    strings = x.split(',')
    ports = []

    for s in strings:
        if '-' in s:
            rng = tuple([int(x) for x in s.split('-', 1)])
            ports.append(rng)
        else:
            ports.append(int(s))

    return ports


if __name__ == '__main__':
    lst = portlist('1000,2000-5000,500-550')
    print(lst)




from __future__ import absolute_import
from os.path import join, dirname
from sokery.web.app import Application
from tornado.ioloop import IOLoop
from sokery.web.admin import LiveHandler,\
					  StaticFileHandler
from sokery.web.proxy import ProxyRequestHandler
from argparse import ArgumentParser
import logging

logging.getLogger().setLevel(logging.INFO)

basepath = dirname(__file__)
webpath = join(basepath, 'web')
viewspath = join(webpath, 'views')
indexpath = join(viewspath, 'index.html')
angularpath = join(viewspath, 'angular.min.js')

urlmap = [
	(r'/live', LiveHandler, ),
	(r'/(.+)$', StaticFileHandler, {'path': viewspath}),
	(r'/(.*)$', StaticFileHandler, {'path': indexpath}),
]

def start_loop(port=4991, listen_to=None, proxy_http=False,
			   allowed_ports=None):
	logging.info('starting in port {}'.format(port))
	logging.info('allowed_ports {}'.format(allowed_ports))
	if proxy_http:
		urlmap.append((r'/proxy', ProxyRequestHandler))

	app = Application(urlmap,
		report_clients=[],
		listen_to=listen_to,
		proxy_http=proxy_http,
		allowed_ports=allowed_ports,
		)
	app.listen(port)
	loop = IOLoop.current()
	loop.start()

if __name__ == '__main__':
	try:
		start_loop()
	except KeyboardInterrupt:
		logging.info('bye')


from __future__ import absolute_import
import tornado.web
import socket
import json
from datetime import datetime
from sokery.web.tcp import TCPServer
from sokery.utils.types import portrange
import logging
import os


class Application(tornado.web.Application):

    def __init__(self, *args, **kwargs):
        super(Application, self).__init__(*args, **kwargs)
        self.servers = {}

    @property
    def report_clients(self):
        return self.settings['report_clients']

    @property
    def listen_to(self):
        return self.settings['listen_to']

    def listen(self, port):
        super(Application, self).listen(port)
        
        # If there are preconfigured ports, just
        # open them from with "open_port" function.
        ports = self.listen_to
        if isinstance(ports, list):
            for port in ports:
                self.open_port(port)

    def open_port(self, port, options=None):
        """ Opens a port and adds it to the stack """

        try:
            if not self.is_port_allowed(port):
                raise Exception('not allowed port')
            
            server = TCPServer()
            server.report_clients = self.report_clients
            server.listen(port)

            self.servers[port] = server

            if options:
                server.echo_all = bool(options.get('echo_all', None))
                server.is_hex_digest = bool(options.get('is_binary', None))
                logging.info("echo all: {}".format(server.echo_all))

            self.report('success',
                msg='listening in port {}'.format(port)
            )
            self.report('new-server',
                msg={
                    'port': port,
                    'server_id': server.uuid,
                    'is_hex_digest': server.is_hex_digest,
                }
            )

        except socket.error as ex:
            if ex.errno == 13:
                self.report('error',
                    msg='cannot open port {}'.format(port)
                )

            elif ex.errno == 48:
                self.report('error',
                    msg='port {} already in use'\
                        .format(port)
                )

            else:
                self.report('error', msg=str(ex))

        except Exception as ex:
            self.report('error', msg=str(ex))

    def is_port_allowed(self, port):
        allow_map = self.settings['allowed_ports']
        
        if not allow_map:
            return True

        for item in allow_map:
            if isinstance(item, int):
                if item == port:
                    return True
            elif isinstance(item, tuple):
                low, high = item
                if port >= low and port <= high:
                    return True

        return False

    def close_port(self, port):
        self.servers[port].stop()
        del self.servers[port]
        self.report('stopped-server', msg={'port': port})

    def report(self, op, msg='error'):
        logging.info("{}: {}".format(op, msg))
        for client in self.report_clients:
            client.write_message(json.dumps({
                'op': op,
                'message': msg,
            }))




from __future__ import absolute_import
import os
import json
from tornado.web import RequestHandler, StaticFileHandler
from tornado.websocket import WebSocketHandler
from sokery.web.tcp import TCPServer


class LiveHandler(WebSocketHandler):

    def open(self):
        clients = self.settings['report_clients']
        clients.append(self)
        self.write_message(json.dumps({
            'op': 'list-servers',
            'msg': {
                'servers': [
                    {'port': port} \
                    for port in self.application.servers
                ]
            }
        }))

    def on_close(self):
        clients = self.settings['report_clients']
        clients.remove(self)

    def on_message(self, message):
        data = json.loads(message)
        op = data['op']

        if op == 'listen':
            port = data['port']
            options = data.get('options', None)
            self.application.open_port(port, options=options)

        elif op == 'unlisten':
            port = data['port']
            self.application.close_port(port)





from tornado.web import RequestHandler
from binascii import hexlify
import requests
import json
import os


class Error(Exception): pass

class ProxyRequestHandler(RequestHandler):

    sokery_methods = ['connect', 'received']

    def post(self):
        report_clients = self.settings['report_clients']
        headers = self.request.headers
        sk_method = headers.get('x-sokery-method')
        app_name = headers.get('x-sokery-app-name')
        ct = headers.get('content-type', None)
        data_recvd = self.request.body
        src_address = headers.get('x-sokery-client-address', None)
        address = src_address.split(':')

        if not app_name:
            self.write('invalid app name')
            return

        if not sk_method in self.sokery_methods:
            self.write('invalid sokery method')
            return

        if len(address) < 3:
            self.write('invalid address')
            return

        # convert ports to number
        address[1] = int(address[1])
        address[2] = int(address[2])

        if ct == 'application/octet-stream':
            # TODO: unify with tcp.py method, it's 
            #       "very" repeated.
            pkg = json.dumps({
                'op': 'recvd',
                'is_binary': False,
                'data': data_recvd,
                'from': address,
                'is_proxied': True,
                'proxy_name': app_name
            })
            for client in report_clients:
                client.write_message(pkg)

        else:
            self.write('error')


class ProxyClient(object):
    __slots__ = ['url', 'app_name']

    def __init__(self, url, app_name=None):
        self.url = url
        if not app_name:
            self.app_name = 'app{}'.format(
                hexlify(os.urandom(3))
            )
        else:
            self.app_name = app_name

    def post_recvd(self, data, address):
        if len(address) < 3:
            raise Error('expecting 3 address params')

        addr = '{}:{}:{}'.format(*address)

        headers = {
            'content-type': 'application/octet-stream',
            'x-sokery-app-name': self.app_name,
            'x-sokery-method': 'received',
            'x-sokery-client-address': addr
        }
        print(addr)
        response = requests.post(self.url,
                                 data=data,
                                 headers=headers,
                                 )
        print(response.content)


if __name__ == '__main__':
    addr = ('127.0.0.1', 12341, 4995)
    client = ProxyClient('http://127.0.0.1:4991/proxy')
    client.post_recvd('SEG1,RPT,00:01:02:03:05:1,2,3,0\n', addr)



from setuptools import setup
import sokery
import os

basepath = os.path.dirname(__file__)
binpath = os.path.join(basepath, 'bin')

setup(
  name = 'sokery',
  packages = ['sokery', 'sokery.web', 'sokery.utils'],  
  long_description=open(os.path.join(basepath, 'README.md')).read(),
  version = sokery.__version__,
  description = 'Socket testing with web interface',
  install_requires=['tornado'],
  scripts = [os.path.join(binpath, 'sokery')],
  package_data = {
    'sokery': ['web/views/index.html', 'web/views/ng.js']
  },
  author = 'Gamaliel Espinoza',
  author_email = 'gamaliel.espinoza@gmail.com',
  url = 'https://github.com/gamikun/sokery',
  keywords = ['sokery'], 
  classifiers = [],
)
